;/*************************************************************************/
;/*                                                                       */
;/*                Institut fr Maschinelle Sprachverarbeitung            */
;/*                     University of Stuttgart, Germany                  */
;/*                         Copyright (c) 1998                            */
;/*                        All Rights Reserved.                           */
;/*                                                                       */
;/*  Permission to use, copy, modify, distribute this software and its    */
;/*  documentation for research, educational and individual use only, is  */
;/*  hereby granted without fee, subject to the following conditions:     */
;/*   1. The code must retain the above copyright notice, this list of    */
;/*      conditions and the following disclaimer.                         */
;/*   2. Any modifications must be clearly marked as such.                */
;/*   3. Original authors' names are not deleted.                         */
;/*   4. It may not be redistributed in a modified version or within      */
;/*      another distribution                                             */
;/*  This software may not be used for commercial purposes without        */
;/*  specific prior written permission from the authors.                  */
;/*                                                                       */
;/*  THE UNIVERSITY OF STUTTGART AND THE CONTRIBUTORS TO THIS WORK        */
;/*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
;/*  ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT   */
;/*  SHALL THE UNIVERSITY OF STUTTGART NOR THE CONTRIBUTORS BE LIABLE     */
;/*  FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES    */
;/*  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN   */
;/*  AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,          */
;/*  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF       */
;/*  THIS SOFTWARE.                                                       */
;/*                                                                       */
;/*************************************************************************/
;/*             Author :  Mark Breitenbcher                              */
;/*-----------------------------------------------------------------------*/
;/*                                                                       */
;/* This file deals with POS tagging for German. Two different tagging    */
;/* methods are available:\\                                               */
;/* 1. No_POS that simply does nothing at all\\                           */
;/* 2. The Treetagger by Helmut Schmid, which is an external package not  */
;/* (yet) included in the opensource version.                             */
;/* In this file all parameters are set that are required for the         */
;/* treetagger.\\                                                         */
;/* In TTS two tagsets are used as defined by the parameter 'pos_tagset':\\ */
;/* 1. stts  (STuttgart TagSet                                            */
;/*  http://www.ims.uni-stuttgart.de/ftp/pub/corpora/stts_guide.ps.gz\\   */
;/* 2. ustts ("underspecified" STTS tagset)                               */
;/*  ustts contains the 11 main categories of STTS                        */
;/*  ADJ, ART, ADV, KO, ITJ, N, CARD, AP, P, PTK, V\\                     */
;/* other tagsets as provided by several lexicons (e.g. bomp, celex)      */
;/* are currently mapped to ustts using a table defined in this file      */ 
;/* (in the post_hooks of the lexicon)                                    */
;/*                                                                       */
;/*=======================================================================*/

;; a pos module that does nothing
(define (No_POS utt) 
  "(No_POS utt) A pos module that does nothing."
utt)

; this parameter holds the internally used tagset (stts or ustts)
(Param.def 'pos_tagset nil)

; this parameter is set to t to avoid loading the tagger twice
(Param.def 'treetagger_loaded nil)

;;;  we set pardir to the directory where our parameterfiles are.  
;;;  this is special for the parameterfile for the extern TreeTagger used
;;;  in German version.
(defvar german_pos_par_file nil
"german_pos_par_file
 parent directory of IMS-specific parameter files. (e.g. the TreeTagger 
 parameter files of the german version.")

(define (Init_POS_German)
  "(Init_POS_German)
Init the German Treetagger"
  (if (equal? german_pos_par_file nil)
      (error "(Init_POS_German): german_pos_par_file is not defined.")
      (if (not (Param.get 'treetagger_loaded)) ;don't init twice
	  (begin
	    (Treetagger_German_Init)
	    (Param.set 'treetagger_loaded t)))))

(define (POS_German utt)
  "(POS_German utt)
uses the German treetagger. If the parameter 'pos_tagset' is set to 'celex
the tags are mapped to the simpler Celex tags. Else the tagset is the STTS tagset."
  (Treetagger_German utt)
  (let ((ts (Param.get 'pos_tagset)))
    (cond
     ((equal? ts "ustts")
      (map_all_postags utt stts_ustts_postab)) 
     ((equal? ts "celex")
      (map_all_postags utt stts_celex_postab)))) ; mapping obsolete
utt)

(define (map_all_postags utt tag_map)
"(map_all_postags utt tag_map
Maps the POS tags using the map 'tag_map' changes the `pos'-feature
appropriately."
  (mapcar
   (lambda (wrd)
     (let ((tag (item.feat wrd 'pos)))
       (item.set_feat wrd 'pos (cdr (assoc tag tag_map)))))
   (utt.relation.items utt 'Word)))


(define (map_pos_celex_ustts entry) 
  "(map_postag_bomp_stts entry
post_hook for lexicon: maps pos tag from celex tagset to (underspec.) stts tagset"
  (let ((tag (cadr entry))) 
    (let ((utag (cdr (assoc tag celex_ustts_postab))))
      (list 
       (car entry) 
       (if utag utag tag); not in map: assume it is already ustts (from addenda)
       (caddr entry)))))

(define (map_pos_bomp_ustts entry) 
  "(map_postag_bomp_stts entry
post_hook for lexicon: maps pos tag from bomp tagset to (underspec.) stts tagset"
  (let ((tag (cadr entry)))
    (let ((utag (cdr (assoc tag bomp_ustts_postab))))
      (list 
       (car entry) 
       (if utag utag tag) ; not in map: assume it is already ustts (from addenda)
       (caddr entry)))))

;; POS tagset mapping tables

(defvar stts_ustts_postab nil
"table to map the STTS tagset to main STTS categories.")
(set! stts_ustts_postab
  '(
    ("ADJA" . "ADJ")
    ("ADJD" . "ADJ")
    ("ADV" . "ADV")
    ("APPR" . "AP")
    ("APPRART" . "AP")
    ("APPO" . "AP")
    ("APZR" . "AP")
    ("ART" . "ART")
    ("CARD" . "CARD") 
    ("FM" . "N") ;?
    ("ITJ" . "ITJ")
    ("KOUI" . "KO")
    ("KOUS" . "KO")
    ("KON" . "KO")
    ("KOKOM" . "KO")
    ("NN" . "N")
    ("NE" . "N")
    ("PDS" . "P")
    ("PDAT" . "P")
    ("PIS" . "P")
    ("PIAT" . "P")
    ("PIDAT" . "P")
    ("PPER" . "P")
    ("PPOSS" . "P")
    ("PPOSAT" . "P")
    ("PRELS" . "P")
    ("PRELAT" . "P")
    ("PRF" . "P")
    ("PWS" . "P")
    ("PWAT" . "P")
    ("PWAV" . "P")
    ("PAV" . "P")
    ("PTKZU" . "PTK")
    ("PTKNEG" . "PTK") 
    ("PTKVZ" . "PTK")
    ("PTKANT" . "PTK")
    ("PTKA" . "PTK")
    ("TRUNC" . "N")
    ("VVFIN" . "V")
    ("VVIMP" . "V")
    ("VVINF" . "V")
    ("VVIZU" . "V")
    ("VVPP" . "V")
    ("VAFIN" . "V")
    ("VAIMP" . "V")
    ("VAINF" . "V")
    ("VAPP" . "V")
    ("VMFIN" . "V")
    ("VMINF" . "V")
    ("VMPP" . "V")
    ("XY" . "N") ;?
    ("$," . "0")
    ("$." . "0")
    ("$(" . "0")
))

(defvar celex_ustts_postab nil
"table to map the POS tags of the Celex lexicon to main STTS categories.")
(set! celex_ustts_postab
  '(
    (A . ADJ)     ; Adjektiv
    (ART . ART)   ; Artikel 
    (ADV . ADV)   ; Adverbien
    (C . KO)      ; Konjunktionen
    (I . ITJ)     ; Interjektionen
    (N . N)       ; Nomina
    (NUM . CARD)  ; Numeralia (aber auch AD~ z.B. bei Ordinalzahlen)
    (PREP . AP)   ; Prpositionen 
    (PRON . P)    ; Pronomina
    (V . V)       ; Verben
    )
)

(defvar bomp_ustts_postab nil
"table to map the POS tags of the BOMP lexicon to main STTS categories.")
(set! bomp_ustts_postab
  '(
    (ADD . AP)   ; Prpositionen 
    (ART . ART)  ; Artikel
    (INJ . ITJ)  ; Interjektionen
    (KNJ . KO)   ; Konjunktionen (aber auch KOUI)
    (MOD . ADV)  ; Modalwrter
    (NAM . N)    ; Eigennamen
    (NEG . PTK)  ; Negation und Affirmation (mit Vorsicht zu geniessen: auch ja, jawohl, danke sind NEG)
    (NOM . N)    ; Nomina
    (NUM . CARD) ; Numeralia (aber auch AD~ z.B. bei Ordinalzahlen)
    (PPR . P)    ; Personalpronomen
    (PRO . P)    ; Pronomina 
    (QAN . ADJ)  ; Attributive Adjektive
    (QPN . ADJ)  ; Prdikative Adjektive 
    (QPV . ADJ)  ; Prdikativ und adverbial verwendbare Adjektive (Adverbien) 
    (QAV . ADV)  ; Reine Adverbien (nicht flektierbar)
    (RFW . P)    ; Reflexifpronomen?
    (SBJ . KO)   ; Subjunktionen 
    (TME . ADV)  ; Prverbien (Tmesis)
    (VEH . V)    ; Verbformen von haben
    (VEM . V)    ; Modalverben
    (VES . V)    ; Verbformen von sein 
    (VEW . V)    ; Verbformen von werden
    (VRB . V)    ; Vollverben 
    )
)

(defvar bomp_stts_postab nil
"table to map the POS tags of the BOMP lexicon to STTS tags 
(full STTS specification, sometimes simply wrong)")
(set! bomp_stts_postab
  '(
    (ADD . APPR)  ; Prpositionen (aber auch APPO, APPRART, APZR)
    (ART . ART)   ; Artikel
    (INJ . ITJ)   ; Interjektionen
    (KNJ . KON)   ; Konjunktionen (aber auch KOUI)
    (MOD . ADV)   ; Modalwrter
    (NAM . NE)    ; Eigennamen
    (NEG . PTKNEG) ; Negation und Affirmation (mit Vorsicht zu geniessen: auch ja, jawohl, danke sind NEG)
    (NOM . NN)    ; Nomina
    (NUM . CARD)  ; Numeralia (aber auch ADJA, ADJD, z.B. bei Ordinalzahlen)
    (PPR . PPER)  ; Personalpronomen
    (PRO . PPER)  ; Pronomina (aber auch PDS, PDAT, PIS, PIAT, PIDAT, PPOSS, PPOSAT, PRELS, PRELAT, PRF, PWS, PWAT, PWAV)
    (QAN . ADJA)  ; Attributive Adjektive
    (QPN . ADJA)  ; Prdikative Adjektive (oder  ADJD)
    (QPV . ADJA)  ; Prdikativ und adverbial verwendbare Adjektive (Adverbien) (oder  ADJD)
    (QAV . ADV)   ; Reine Adverbien (nicht flektierbar)
    (RFW . PRF)   ; Reflexifpronomen?
    (SBJ . KOUS)  ; Subjunktionen (aber auch KOUI)
    (TME . PTKVZ) ; Prverbien (Tmesis)
    (VEH . VAFIN) ; Verbformen von haben(aber auch VAINF, VAPP)
    (VEM . VMFIN) ; Modalverben (aber auch VMINF)
    (VES . VAFIN) ; Verbformen von sein (aber auch VAINF, VAPP)
    (VEW . VAFIN) ; Verbformen von werden (aber auch VAINF, VAPP)
    (VRB . VVFIN) ; Vollverben (aber auch haeufig VVPP, oder auch VVINF)
    )
)

(defvar stts_celex_postab nil
"table to map the STTS tags to the CELEX tags (obsolete)")
(set! stts_celex_postab
  '(
    ("ADJA" . "A")
    ("ADJD" . "A")
    ("ADV" . "ADV")
    ("APPR" . "PREP")
    ("APPRART" . "PREP")
    ("APPO" . "PREP")
    ("APZR" . "PREP")
    ("ART" . "ART")
    ("CARD" . "NUM") ;A?
    ("FM" . "N") ;?
    ("ITJ" . "I")
    ("KOUI" . "C")
    ("KOUS" . "C")
    ("KON" . "C")
    ("KOKOM" . "ADV")
    ("NN" . "N")
    ("NE" . "N")
    ("PDS" . "PRON")
    ("PDAT" . "PRON")
    ("PIS" . "PRON")
    ("PIAT" . "PRON")
    ("PIDAT" . "PRON")
    ("PPER" . "PRON")
    ("PPOSS" . "PRON")
    ("PPOSAT" . "PRON")
    ("PRELS" . "PRON")
    ("PRELAT" . "PRON")
    ("PRF" . "PRON")
    ("PWS" . "PRON")
    ("PWAT" . "PRON")
    ("PWAV" . "PRON")
    ("PAV" . "ADV")
    ("PTKZU" . "ADV")
    ("PTKNEG" . "PTKNEG") ; exception we want 'nicht' to be special
    ("PTKVZ" . "ADV")
    ("PTKANT" . "ADV")
    ("PTKA" . "ADV")
    ("TRUNC" . "N")
    ("VVFIN" . "V")
    ("VVIMP" . "V")
    ("VVINF" . "V")
    ("VVIZU" . "V")
    ("VVPP" . "V")
    ("VAFIN" . "V")
    ("VAIMP" . "V")
    ("VAINF" . "V")
    ("VAPP" . "V")
    ("VMFIN" . "V")
    ("VMINF" . "V")
    ("VMPP" . "V")
    ("XY" . "N") ;?
    ("$," . "0")
    ("$." . "0")
    ("$(" . "0")
))

(provide 'ims_german_POS)









