;/*************************************************************************/
;/*                                                                       */
;/*                Institut fr Maschinelle Sprachverarbeitung            */
;/*                     University of Stuttgart, Germany                  */
;/*                         Copyright (c) 1998                            */
;/*                        All Rights Reserved.                           */
;/*                                                                       */
;/*  Permission to use, copy, modify, distribute this software and its    */
;/*  documentation for research, educational and individual use only, is  */
;/*  hereby granted without fee, subject to the following conditions:     */
;/*   1. The code must retain the above copyright notice, this list of    */
;/*      conditions and the following disclaimer.                         */
;/*   2. Any modifications must be clearly marked as such.                */
;/*   3. Original authors' names are not deleted.                         */
;/*   4. It may not be redistributed in a modified version or within      */
;/*      another distribution                                             */
;/*  This software may not be used for commercial purposes without        */
;/*  specific prior written permission from the authors.                  */
;/*                                                                       */
;/*  THE UNIVERSITY OF STUTTGART AND THE CONTRIBUTORS TO THIS WORK        */
;/*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
;/*  ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT   */
;/*  SHALL THE UNIVERSITY OF STUTTGART NOR THE CONTRIBUTORS BE LIABLE     */
;/*  FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES    */
;/*  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN   */
;/*  AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,          */
;/*  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF       */
;/*  THIS SOFTWARE.                                                       */
;/*                                                                       */
;/*************************************************************************/
;/*             Author :  Mark Breitenbuecher, Gregor Mhler              */
;/*-----------------------------------------------------------------------*/
;/*                                                                       */
;/* Contains postlexical rules to solve problems concerning the lexicon:\\*/
;/* 1. transforms some ich to ach-sounds (for Celex dict. only)\\         */
;/* 2. joins affricates \\                                                */
;/* 3. prevents some unknown diphones (for Celex dict. only,              */ 
;/* should not be here but in diphone module)\\                           */
;/* 4. give syllables names (transcription of phones in syllables)\\      */
;/* 5. inserts glottal stops of length glottal_stop_len                   */
;/*  (for Celex dict. only, default:0.001s) \\                            */
;/* 6. maps R to aschwa at the end of syllables  \\                       */
;/*=======================================================================*/

(defvar glottal_stop_len 0.001
  "the length of an inserted glottal stop")

(define (german_postlex utt)
  "(german_postlex UTT)
   applies some German postlexical rules on the utterance UTT."
  (begin
    (mapcar
     (lambda (seg)
       (begin
	 (join_affricates seg)
	 (transform_schwa seg)
	 ))
     (reverse (utt.relation.leafs utt 'Segment)))
    (Name_Syls utt)
    (if (equal? 0 (Param.get 'lex_with_glottalstop))
	(begin
	  (Glottalstop_before_Vowel utt)
	  (Glottalstop_after_Prefix utt)))
   utt))

(define (german_postlex_old utt)
  "(german_postlex UTT)
   applys some postlexical rules on the utterance UTT.
   It maps Celex Sampa transcription to german festival phoneset."
  (begin
    (mapcar
     (lambda (seg)
       (begin
	 (sampa2germansampa seg)
	 (transform_ich-ach seg) 
	 (join_affricates seg)
	 (transform_schwa seg)
	 (prevent_unknown_diphones seg) ;should finally go into the diphone selection module!
	 ))
     (reverse (utt.relation.leafs utt 'Segment)))
    (Name_Syls utt)
    (Glottalstop_before_Vowel utt)
    (Glottalstop_after_Prefix utt)
 utt))

(define (join_affricates seg)
  "(join_affricates SEG)
  joins fricatives p f -> pf in segment relation.
  (It uses ``affricate_join_tab''.)"
  
  (let ((affric 
	 ;; differnt syllables?
	 (if (string-equal
	      (item.feat seg 'R:SylStructure.n.name) "0") 
	     ;;yes->
	     nil
	     ;;no->join
	     (list (item.feat seg 'name)
		   (item.feat seg 'n.name))))
	(erg nil))
    ;     (format t "%l %l %l\n" 
    ;	     (item.feat seg 'name) 
    ;	     (if (equal? nextseg nil)
    ;		 nil
    ;		 (item.name nextseg)) 
    ;	     (item.feat seg 'nn.name))
    
    (if (null? (set! erg (cdr (assoc affric affricate_join_tab))))
	nil
	(begin
	  ;;replace first part with affricate
	  (item.set_name seg erg)
	  ;;delete second part
	  (item.delete  (item.next seg))))))    

(defvar affricate_join_tab nil
"affricate_join_tab
is an assoc list that defines the mapping from two phonemes to one
for example t s -> ts (not accros syllable boundaries!)")
(set! affricate_join_tab
  '(
    (("t" "Z") . "tZ")
    (("p" "f") . "pf")
    (("t" "s") . "ts")
    (("t" "S") . "tS")
    (("t" "Z") . "tS")
))

(define (transform_ich-ach seg)
  "(transform_ich-ach SEG)
  transforms Ich-Phoneme (C)  in Ach-Phoneme (x) in segment relation
  with the help of the ``ich_ach_tab'' (because Celex dosen't 
  destinguish between x and C)."
  (let ((ichseq 	 
	 ;; same word?
	 (if (and
	      (string-equal
	       (item.feat seg 'R:SylStructure.n.name) "0")
	      (string-equal
	       (item.feat seg 'R:SylStructure.parent.n.name) "0"))
	     nil 
	     ;;no->join
	     (list (item.feat seg 'name)
		   (item.feat seg 'n.name))))
	(erg nil))
    (if (null? (set! erg (cdr (assoc ichseq ich_ach_tab))))
	nil
	(begin
	  (item.set_name (item.next seg) (cadr erg))))))


(define (sampa2germansampa seg)
  "(sampa2germansampa SEG)
  translates Celex SAMPA to german SAMPA (with
  the help of ``sampa2germansampa_tab'')"
  (let ((sampa (item.name seg))
	(erg nil))
    (if (null? (set! erg (cdr (assoc sampa  sampa2germansampa_tab))))
	nil
	(item.set_name seg erg))))

(define (transform_schwa seg)
  "(transform_schwa SEG)
  Deals with words containing a ``Schwa'' with the
  help of ``schwa_tab'' (like in 'der', 'wert')"
  (let ((phones
	 ;; differnt syllables?
	 (if (string-equal
	      (item.feat seg 'R:SylStructure.n.name) "0") 
	     ;;yes->
	     nil
	     ;;no->join
	     (list (item.feat seg 'name)
		   (item.feat seg 'n.name))))
	(erg nil))
    (if (null? (set! erg (assoc phones schwa_tab)))
	nil
	(begin
	  ;;replace first part with first translated
	  (item.set_name seg (cadr erg))
	  ;;delete second part
	  (if (item.next seg)
	      (if (null? (cddr erg))
		  (item.delete  (item.next seg))    
		  (item.set_name (item.next seg) (caddr erg))))))))



(defvar schwa_tab nil
"schwa_tab
is an assoc list that defines the mapping from two phonemes to 
two others (not accros syllable boundaries!)")
(set! schwa_tab
  '(
    (("i:" "R") . ("i:" "6"))  
    (("I" "R")  . ("I" "6"))  
    (("y:" "R") . ("y:" "6")) 
    (("Y" "R")  . ("Y" "6"))  
    (("e:" "R") . ("e:" "6"))  ;  e.g. 'der', 'wert'
    (("E" "R")  . ("E" "6"))   ;  e.g. 'Verstaendnis'
    (("E:" "R") . ("E:" "6")) 
    (("2:" "R") . ("2:" "6")) 
    (("9" "R")  . ("9" "6"))  
    (("u:" "R") . ("u:" "6")) 
    (("U" "R")  . ("U" "6"))  
    (("o:" "R") . ("o:" "6")) 
    (("O" "R")  . ("O" "6"))  
    (("a:" "R") . ("a:" "6")) 
    (("a" "R")  . ("a" "6"))  
    (("aI" "R") . ("aI" "6")) 
    (("OY" "R") . ("OY" "6")) 
    (("aU" "R") . ("aU" "6")) 
    (("eI" "R") . ("eI" "6")) 
    (("@" "R")  . ("6"))       
    (("h" "@")  . ("@"))       ; e.g. 'hohe'
))

(defvar  sampa2germansampa_tab nil
"sampa2germansampa_tab nil
is an assoc list that defines the mapping from Celex SAMPA to german SAMPA
(as used in festival)")

(set! sampa2germansampa_tab
  '(("r"  ."R");
    ;; vowels
    ("A:"."a:");
    ("O:"."O" );
    ("3:"."2:");
    ("w"."v");
    ("&:"."2:");
    ("Q" ."9" );
    ("&" ."@" );
    ("A" ."a" );
    ("V" ."a" );
    ;; diphtons
    ("eI"."e:");     ;; english import
    ("ai"."aI");
    ("au"."aU");
    ("OI"."OY");
    ("Oy"."OY");
    ;; nasals & other imports
    ("dZ" ."tS" );
    ;;    ("Q~:"."y:" );   ;; German "equiv."
    ("Q~:" ."9^" );
    ;;    ("A~:"."O N");   ;; German "equiv."
    ("A~:" ."a^" );
    ;;    ("&~:"."E:" );   ;; German "equiv."
    ("&~"  ."E^" );
    ;;    ("O~:"."O"  );   ;; German "equiv."
    ("O~:" ."o^" );
    ;;    ("&~" ."@"  );   ;; German "equiv."
    ("&~"  ."E~" );

    ;; ch-sounds (also across syllable boundaries)
    ("x"   ."C");
))

(defvar ich_ach_tab nil
  "ich_ach_tab
   is an assoc list that defines the mapping from ich-phonemes to ach-phoneme
   (across syllable boundaries!)")

(set! ich_ach_tab
  '(
    (("a:" "C").("a:" "x")) 
    (("a" "C") .("a" "x") ) 
    (("o:" "C").("o:" "x")) 
    (("O" "C") .("O" "x") ) 
    (("u:" "C").("u:" "x")) 
    (("U" "C") .("U" "x") ) 
    (("aU" "C").("aU" "x")) 
))

(define (Glottalstop_before_Vowel utt) 
  "(Glottalstop_before_Vowel UTT)
   Adds a glottal stop before a word beginning with a vowel."
  (mapcar
   (lambda (seg)
     (if (and (not (string-equal (item.feat seg 'n.R:SylStructure.parent.parent.name) 
				 (item.feat seg 'R:SylStructure.parent.parent.name)))
	      ;(not (string-equal (item.name seg) "_"))
	      (string-equal (item.feat seg 'n.ph_vc) "+"))
	   (insert_glottal_stop_after seg)))   
	   ;;(set! itemin (item.insert seg (list "?") 'before)))))
	   ;;(item.relation.insert seg 'SylStructure itemin 'before))))
   (utt.relation.items utt 'Segment))
  utt)

(define (Glottalstop_after_Prefix utt)
  "(Glottalstop_after_Prefix UTT)
   Adds a glottal stop between certain prefixes (ent-,um-,aus-,dis-,un-,mis-,
   ver-,in-,ge-,co-,be-,ab-) and syllable beginning with a vowel. 
   Should actually work on morphemes."
  (mapcar
   (lambda (seg)
     (if (and (or 
	       (string-matches 
		(item.feat seg 'R:SylStructure.parent.name) 
		"\\(Ent\\|Um\\|aUs\\|dIs\\|dEs\\|Un\\|mIs\\|fER\\|In\\|g@\\|ko:\\|b@\\|ap\\)")
		
	       (and (string-matches (item.feat seg 'R:SylStructure.parent.name) "d@R")
		       (string-matches (item.feat seg 'R:SylStructure.parent.p.name) "vi:")))
	      (string-equal (item.feat seg 'n.ph_vc) "+")
	      (not (string-equal (item.feat seg 'R:SylStructure.parent.name) (item.feat seg 'n.R:SylStructure.parent.name))))
	   (insert_glottal_stop_after seg)))
   (utt.relation.items utt 'Segment))
  utt)


(defvar unknown_diphones_tab nil
"unknown_diphones_tab nil
is an assoc list that defines which diphones don't exist and possible modifications.
If no modifications are given an glottal stop will be inserted.")

(set! unknown_diphones_tab
  '(
    (("R" "j") . 1) 
    (("b" "a^") . 1) 
;    (("h" "@") . ("h" "E")) 
))

(define (prevent_unknown_diphones seg)
  "(prevent_unknown_diphones seg)
  Between all diphone-combinations that are defined in unknown_diphones_tab
  as well as duplicated phones, we will insert a glottal stop."
  (let ((name (item.feat seg 'name))
	(nname (item.feat seg 'n.name)))
    (let ((rdiph (cdr (assoc (list name nname) unknown_diphones_tab  ))))
      (cond
       ((or (equal? name nname)
	    (equal? 1 rdiph))
	(format t "inserting ? between /%s/ and /%s/.\n" name nname)
	(insert_glottal_stop_after seg))
       ((equal? 2 (length rdiph))
	(let ((nseg (item.next seg)))
	  (item.set_feat seg 'name (car rdiph))
	  (if nseg (item.set_feat nseg 'name (cadr rdiph)))
	  (format t "replacing /%s,%s/ by /%s,%s/.\n" name nname (car rdiph) (cadr rdiph))))
       )))
  t)

(define (insert_glottal_stop_after seg)
  "insert a glottal stop after this segment. 
  Its length is controlled by the variable 'glottal_stop_length'."

  (item.set_feat (item.insert seg (list "?") 'after) 'end 
		 (+ glottal_stop_len (item.feat seg 'end))))

(define (insert_glottal_stop_after seg)
  "insert a glottal stop after this segment. 
  Its length is controlled by the variable 'glottal_stop_length'."
  (let ((nseg (item.insert seg nil 'after)))
    (item.set_feat nseg 'end 
		   (+ glottal_stop_len (item.feat seg 'end)))
    (item.set_feat nseg 'name "?"))) 

; not necessary: see (transform_schwa seg)
;; change R in syllable coda to a_schwa ( fER - StEnt - nIs -> f6 - StEnt - nis,
;; wIRt -> wI6t  AS 02/22/00
(define (map_R_to_aschwa utt)
"(map_R_to_aschwa UTT)
maps R at end of syllables to a_schwa
modifications of segments by this function are
not visible in the Syllable relation: 
name of syllable remains unchanged"

(let ((syls (utt.relation.items utt 'Syllable)))
  (while syls
	 (let ((seg (item.daughtern (item.relation (car syls) 'SylStructure))))
	   (if (string-equal "R" (item.name seg))
;; seg is R: change to 6, if following seg is not @ or 6 (we don't have diphones for this case)
               (if (or (null (item.daughter1 (item.next (item.parent (item.relation seg 'SylStructure)))))
                       (not (string-matches (item.name (item.daughter1 (item.next (item.parent (item.relation seg 'SylStructure))))) "@\\|6")))
                   (progn
                    (item.set_name (item.daughtern (item.relation (car syls) 'SylStructure)) "6")
                    (if (and (string-equal "E" (item.name (item.prev seg)))
                             (equal? 0 (item.feat (car syls) 'stress)))
;; preceding seg was E, syl is unstressed; delete E ( ER -> 6 ), unless it's at the beginning of a phrase: we can't delete it there, because we don't have a diphone _-6
                        (if (not (null (item.prev (item.prev seg))))
                            (if (not (string-equal "_" (item.name (item.prev (item.prev seg)))))
                                (item.delete (item.prev seg)))))))
               (if (not (null (item.prev seg)))
                   (if (and (string-equal "R" (item.name (item.prev seg)))
                            (string-equal "-" (item.feat seg 'ph_vc)))
                       (item.set_name (item.prev seg) "6"))))
               
           (set! syls (cdr syls))))))


(provide 'ims_german_postlex)















