;/*************************************************************************/
;/*                                                                       */
;/*                Institut fr Maschinelle Sprachverarbeitung            */
;/*                     University of Stuttgart, Germany                  */
;/*                         Copyright (c) 1998                            */
;/*                        All Rights Reserved.                           */
;/*                                                                       */
;/*  Permission to use, copy, modify, distribute this software and its    */
;/*  documentation for research, educational and individual use only, is  */
;/*  hereby granted without fee, subject to the following conditions:     */
;/*   1. The code must retain the above copyright notice, this list of    */
;/*      conditions and the following disclaimer.                         */
;/*   2. Any modifications must be clearly marked as such.                */
;/*   3. Original authors' names are not deleted.                         */
;/*   4. It may not be redistributed in a modified version or within      */
;/*      another distribution                                             */
;/*  This software may not be used for commercial purposes without        */
;/*  specific prior written permission from the authors.                  */
;/*                                                                       */
;/*  THE UNIVERSITY OF STUTTGART AND THE CONTRIBUTORS TO THIS WORK        */
;/*  DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING      */
;/*  ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT   */
;/*  SHALL THE UNIVERSITY OF STUTTGART NOR THE CONTRIBUTORS BE LIABLE     */
;/*  FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES    */
;/*  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN   */
;/*  AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,          */
;/*  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF       */
;/*  THIS SOFTWARE.                                                       */
;/*                                                                       */
;/*************************************************************************/
;/*         Author :  Antje Schweitzer (schweiae@ims.uni-stuttgart.de)    */
;/*         Date   :  April 1999                                          */
;/*-----------------------------------------------------------------------*/
;/*                                                                       */
;/* This file provides the module \emph{Ling_Preprocessing} that deals    */
;/* with linguistic text preprocessing. To include it in the synthesis    */
;/* process, the function                                                 */
;/* \texttt{redefine_UttTypes_for_ling_preproc,} as defined in            */
;/* \texttt{ims_german_scm,} must be called (usually in                   */
;/* \texttt{ims_german_voices}).                                          */
;/*                                                                       */
;/* The linguistic analysis carried out within the module creates an      */
;/* additional relation \texttt{NP} for each utterance containing all     */
;/* noun chunks of the utterance. These can be used for the prediction of */
;/* accents and phrase boundaries, together with a number of other        */
;/* linguistic features and functions defined in                          */
;/* \texttt{ims_german_ling_preproc}. The most important function is      */
;/* the identification of the so-called 'Vorfeld', i.e. enumerations and  */
;/* focus particles, as well as 'weight' and 'head' of noun chunks and    */
;/* the analysis of sequences of verbs.                                   */
;/*                                                                       */
;/* CART trees for the prediction of phrase breaks                        */
;/* (\texttt{german_rule_based_phrase_cart_tree} and                      */
;/* \texttt{german_data_based_phrase_cart_tree}) and accents              */
;/* (\texttt{german_rule_based_accent_cart_tree} and                      */
;/* \texttt{german_data_based_accent_cart_tree}) using the                */
;/* linguistic features introduced by \texttt{Ling_Preprocessing} are     */
;/* included in \texttt{ims_german_ling_preproc}. To use them,            */
;/* \texttt{phrase_cart_tree} and \texttt{int_accent_cart_tree}           */
;/* must be set accordingly (e.g. in \texttt{ims_german_voices}).         */
;/*                                                                       */
;/*             (Based on tobi_f0.scm by Alan W Black)                    */
;/*                                                                       */
;/*=======================================================================*/


(defvar printdebug 0
  "printdebug
   is a verbosity level that prints debug information. If set
   to 1 the accent name and location is printed. More information
   for higher values")

; use this method or take the commands into the voice definition
(define (setup_tobi_f0_method)
  "(setup_tobi_f0_method)
Set up parameters for current voice to use the implementaion
of ToBI labels to F0 targets by rule."

  (Param.set 'Int_Method 'ToBI)  
  (set! int_accent_cart_tree simple_int_accent_cart_tree) 
  (set! int_tone_cart_tree   simple_int_tone_cart_tree) 

  (Param.set 'Int_Target_Method 'Int_Targets_General)
  (set! int_general_params
	(list 
	 (list 'targ_func tobi_f0_targets)))   ; this causes the function tobi_f0_targets to be used in case Int_Target_Method is Int_Targets_General
;  (set! phrase_cart_tree tobi_label_phrase_cart_tree) ; redefines the phrasebreak tree
  )

;;; Set global parameters

  (Param.set 'Default_Start_Topline 120) ;110
  (Param.set 'Default_End_Topline 90) ;100
  (Param.set 'Default_Start_Baseline 100) ;90
  (Param.set 'Default_End_Baseline 80)   ;80
  (Param.set 'Current_Start_Topline (Param.get 'Default_Start_Topline))
  (Param.set 'Current_End_Topline (Param.get 'Default_End_Topline))
  (Param.set 'Current_Start_Baseline (Param.get 'Default_Start_Baseline))
  (Param.set 'Current_End_Baseline (Param.get 'Default_End_Baseline))
  (Param.set 'Downstep_Factor 0.70)
  (Param.set 'Current_Downstep 1)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (tobi_f0_targets utt syl)
 "(tobi_f0_targets UTT SYL)
 returns a list of time/f0-values for a given syllable"


 (if (>= printdebug 2)
    (format t "Entering tobi_f0_targets for %l \n" (item.name syl)))
 
 (if (equal? 
      (item.relation (item.daughter1 (item.relation (item.daughter1 (item.parent (item.relation (item.parent (item.relation syl 'SylStructure)) 'Phrase))) 'SylStructure)) 'Syllable) syl)
; syl is first Syllable of Phrase: Set Parameters
     (progn
      (Param.set 
       'Phrase_Start 
       (item.feat syl 'R:SylStructure.parent.R:Phrase.first.word_start))
      (Param.set 
       'Phrase_End 
       (item.feat syl 'R:SylStructure.parent.R:Phrase.last.word_end))
      (Param.set 
       'Current_Start_Topline (Param.get 'Default_Start_Topline))
      (Param.set
       'Current_End_Topline (Param.get 'Default_End_Topline))
      (Param.set 
       'Current_Start_Baseline (Param.get 'Default_Start_Baseline))
      (Param.set 
       'Current_End_Baseline (Param.get 'Default_End_Baseline))
      ; reset downstep even after ip-boundary
      (Param.set 'Current_Downstep 1)
      (if (>= printdebug 2)
	  (format t "New parameters: \n Phrase_Start: %f \n Phrase_End: %f \n Start_Topline: %f \n End_Topline: %f \n Start_Baseline: %f \n End_Baseline: %f \n Current_Downstep: %f \n"
		  (Param.get 'Phrase_Start)
		  (Param.get 'Phrase_End)
		  (Param.get 'Current_Start_Topline)
		  (Param.get 'Current_End_Topline)
		  (Param.get 'Current_Start_Baseline)
		  (Param.get 'Current_End_Baseline)
		  (Param.get 'Current_Downstep)))
      
      ()))
 
  (cond
   
   ((null (string-equal "NONE" (item.feat syl 'lisp_german_accent)))

; syllable is accented
; in case there's also an endtone on syl,
; targets for the endtone are determined by german_accent_tree as well
   (progn
    (if (>= printdebug 1)
    (if (string-equal (item.feat syl 'lisp_german_endtone) "NONE")
	(format stderr "### %l (%.2f %.2f) %l \n" (item.name syl)
		(item.feat syl 'syllable_end)
		(item.feat syl 'syllable_start)
		(item.feat syl 'lisp_german_accent))
	(format stderr "### %l (%.2f %.2f) %l, %l \n" (item.name syl)
		(item.feat syl 'syllable_end)
		(item.feat syl 'syllable_start)
		(item.feat syl 'lisp_german_accent)
		(item.feat syl 'lisp_german_endtone))))
	
    (target_set_to_values syl (wagon syl german_accent_tree))))

  ((null (string-equal "NONE" (item.feat syl 'lisp_german_endtone)))

; syl is not accented, but there's an endtone on syl
   (progn
    (if (>= printdebug 1)
    (format stderr "### %l (%.2f %.2f) %l \n" (item.name syl)
	    (item.feat syl 'syllable_end)
	    (item.feat syl 'syllable_start)
	    (item.feat syl 'lisp_german_endtone)))

    (target_set_to_values syl (wagon syl german_endtone_tree))))

  ((or (null (item.prev syl))
       (string-matches (item.feat (item.prev syl) 'lisp_german_endtone) ".*\%.*"))
; syl is first syl in intonation phrase, set low target

    (progn
     (if (>= printdebug 1)
         (format stderr "### low target: %l (%.2f %.2f) %l\n" (item.name syl)
                 (item.feat syl 'syllable_end)
                 (item.feat syl 'syllable_start)
                 (item.feat syl 'lisp_german_endtone)))
     (target_set_to_values syl '(((A 0 0))))))
  
  (t ())))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (target_set_to_values syl targetlist)
 "(target_set_to_values SYL TARGETLIST)
  takes a list of relative targets for a syllable
  and returns a list of absolute (time/frequency)-values"

(if (>= printdebug 2)
    (format t "Entering target_set_to_values with %l \n" targetlist))
	
(cond 
 ((null targetlist) ())
 ((string-equal "UNKNOWN" (car (car (car targetlist))))
  (progn
   (format t "*** Unknown accent found for %l*** \n" (item.name syl))
   ()))
 ((string-equal "ERROR" (car (car (car targetlist))))
  (progn
   (format t "*** This sequence of accents/boundaries is not implemented! (%l) ***\n" (item.name syl))
   ()))

;; get voicing times for syllable

 (t (let 	((pvoicing ())
		 (voicing (get_voicing_time syl))
		 (nvoicing ()))
      (if (not (null (item.prev syl))) 
	  (set! pvoicing (get_voicing_time (item.relation.prev syl 'Syllable))))
      (if (not (null (item.next syl))) 
	  (set! nvoicing (get_voicing_time (item.relation.next syl 'Syllable))))
      
;; get absolute values

      (target_to_value syl targetlist voicing pvoicing nvoicing)))))
 

  

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (get_voicing_time syl)
 "(get_voicing_time SYL)
 returns starttime and duration of voiced segments
 of a given syllable"

(if (>= printdebug 2)
    (format t "Voicing start and duration for %l \n" (item.name syl)))

(cond
 ((null syl) nil)
; get first voiced segment in syllable, starting from first segment
 ((let ((first_voiced 
	 (next_voiced_segment syl (item.relation.daughter1 syl 'SylStructure))))
; get start_time of first voiced segment
; and duration of voicing 
    (list (item.feat first_voiced 'segment_start)
	  (get_voicing_duration syl first_voiced))))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (next_voiced_segment syl seg)
 "(next_voiced_segment SYL SEG)
 returns starttime of first voiced segment in syllable, 
 starting from seg; this function is called by (get_voicing_time SYL)"

(if (>= printdebug 4)
    (format t "Entering next_voiced_segment for %l \n" (item.name seg)))

(cond
; segment in next syllable: something's wrong
 ((null (equal? (item.feat seg 'R:SylStructure.parent.name) (item.name syl))) 
  (print "next_voiced_segment: no voiced segment found."))
; voiced segment: return segment
 ((or (string-equal "+" (item.feat seg 'ph_vc)) 
      (string-equal "+" (item.feat seg 'ph_cvox)))
  seg)
; voiceless segment: check next segment
 ((not (null (item.next seg))) (next_voiced_segment syl (item.next seg)))
; no following segment: something's wrong
 (t (print "next_voiced_segment: no voiced segment found."))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (get_voicing_duration syl seg)
 "(get_voicing_duration SYL SEG)
 adds up duration of voiced segments in syl, starting from seg;
 this function is called by (get_voicing_time SYL)"

(if (>= printdebug 4)
    (format t "Entering get_voicing_duration for %l \n" (item.name seg)))

(cond
; segment in next syllable: don't add duration and end
 ((null (equal? (item.feat seg 'R:SylStructure.parent.name) 
		    (item.name syl)))
  0)
; voiceless segment: don't add duration and end
 ((and (not (string-equal "+" (item.feat seg 'ph_vc)))
       (not (string-equal "+" (item.feat seg 'ph_cvox))))
  0)
; no following segment: add duration and end
 ((null (item.next seg)) (item.feat seg 'segment_duration))
; voiced segment: add duration
 (t (+ (item.feat seg 'segment_duration) 
       (get_voicing_duration syl (item.next seg))))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (target_to_value syl targets voicing pvoicing nvoicing)
 "(target_to_value SYL TARGETS VOICING PVOICING NVOICING)
  gets actual targets for a list of target triples (TARGETS), 
  each triple consisting in a letter (P for previous syllable, 
  A for actual syllable, N for next);
  voicing start and duration for actual, previous and next 
  syllable are given in VOICING, PVOICING, NVOICING, respectively"

(if (>= printdebug 2)
    (format t "Entering target_to_value for %l \n" targets))

(cond
 ((eq 1 (length (car targets)))

; single target
  (cond
   ((string-equal "P" (car (car (car targets))))
   ; target for previous syllable
    (let ((time (get_target_time pvoicing (car (cdr (car (car targets)))))))
    (list (list time (get_frequency time (car (cdr (cdr (car (car targets))))))))))
   ((string-equal "A" (car (car (car targets))))
   ; target for this syllable
    (let ((time (get_target_time voicing (car (cdr (car (car targets)))))))
    (list (list time (get_frequency time (car (cdr (cdr (car (car targets))))))))))
   ((string-equal "N" (car (car (car targets))))
   ; target for next syllable
    (let ((time (get_target_time nvoicing (car (cdr (car (car targets)))))))
    (list (list time (get_frequency time (car (cdr (cdr (car (car targets))))))))))))

; target list
 (t (append (target_to_value syl 
			     (list (list (car (car targets)))) 
			     voicing pvoicing nvoicing)
	    (target_to_value syl (list (cdr (car targets)))
			     voicing pvoicing nvoicing)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (get_target_time voicing rel_time)
 "(get_target_time VOICING REL_TIME)
 given relative time for target and voicing time,
 this function gets the absolute time of the target"

(if (>= printdebug 2)
    (format t "Entering get_target_time with voicing %l, rel_time %l \n"
	    voicing rel_time))

(+ (car voicing) 
   (* (/ rel_time 100) 
      (car (cdr voicing)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (get_frequency time rel_frequency)
 "(get_frequency TIME REL_FREQUENCY)
 given relative frequency of target and absolute target time,
 this function gets the absolute frequency value of the target"

(if (>= printdebug 2)
    (format t "Entering get_frequency with time %l, rel_frequency %l \n"
	    time rel_frequency))

(cond

 ((string-equal "DSTEP" rel_frequency)

  (let ((base (ttt_get_current_baseline time))
	(top (ttt_get_current_topline time)))

; Downstep declines range of following targets: 
; set new Downstep_Factor

    (Param.set 'Current_Downstep 
		   (* (Param.get 'Current_Downstep) 
		      (Param.get 'Downstep_Factor)))
    (format t "Downstep: Current_Downstep now %f \n" (Param.get 'Current_Downstep))

; get absolute frequency, taking into account 'Current_Downstep

    (+ base (* (- top base) 
	       (Param.get 'Current_Downstep)))))


 ((number? rel_frequency)

  (let ((base (ttt_get_current_baseline time))
	(top (ttt_get_current_topline time)))

; get absolute frequency, taking into account 'Current_Downstep

    (+ base (* (- top base)
	       (* (/ rel_frequency 100) (Param.get 'Current_Downstep))))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (ttt_get_current_baseline v)
  "Returns the current declined baseline at time v."
  (if (>= printdebug  4) (begin
      (print "Entering  ttt_get_current_baseline with:")
      (pprintf v)))
  (let ((h (Param.get 'Current_Start_Baseline))
	(l (Param.get 'Current_End_Baseline))
	(e (Param.get 'Phrase_End))
	(s (Param.get 'Phrase_Start)))
    (- h (* (/ (- h l) (- e s)) (- v s)))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (ttt_get_current_topline v)
  "Returns the current declined topline at time v."
  (if (>= printdebug  4) (begin
      (print "Entering  ttt_get_current_topline with:")
      (pprintf v)))
  (let ((h (Param.get 'Current_Start_Topline))
	(l (Param.get 'Current_End_Topline))
	(e (Param.get 'Phrase_End))
	(s (Param.get 'Phrase_Start)))
    (- h (* (/ (- h l) (- e s)) (- v s)))))



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; CART trees

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; CART tree to specify no accents

(set! no_int_cart_tree
'
((NONE)))


;;; a very simple CART tree for accent prediction
; should work with stts and ustts tags (i.e. with and without tree tagger)
(set! simple_int_tone_cart_tree
'
      ((lisp_last_syl_in_word is "1")
       ((R:SylStructure.parent.R:Word.pbreak is "BB")
	((R:SylStructure.parent.lisp_german_end_punc matches "\\?")
	 ((lisp_first_word_in_sentence matches "^PW.*")
	  (("%"))
	  (("H%")))
	 ((R:SylStructure.parent.lisp_german_end_punc matches ";")
          (("H%"))
          ((R:SylStructure.parent.lisp_german_end_punc matches ",")
           ((lisp_pos_after_next_break matches "^KOUS\\|^KON")
            (("H%"))
            (("H%")))
           ((R:SylStructure.parent.top matches "Vorfeld_Ende")
            (("H%"))
            (("%")))
           (("%")))))
	((R:SylStructure.parent.R:Word.pbreak is "B")
	 (("-"))
	 ((NONE))))
       ((NONE))))

(set! even_simpler_int_tone_cart_tree
'
      ((lisp_last_syl_in_word is "1")
       ((R:SylStructure.parent.R:Word.pbreak is "BB")
	((R:SylStructure.parent.R:Token.parent.punc matches ".*\\?.*")
	 ((lisp_first_word_in_sentence matches "^PW.*")
	  (("%"))
	  (("H%")))
	 (("%")))
	((R:SylStructure.parent.R:Word.pbreak is "B")
	 (("-"))
	 ((NONE))))
       ((NONE))))



; a simple cart tree to set tobi accents on the stressed syl of content words 
(set! simple_int_accent_cart_tree
;;; H*L auf NN/NE/V*/ITJ vor (vollen) Phrasengrenzen
;;; L*H auf alle uebrigen NN/NE/V*/ITJ
      '
;;; nur auf NNs/NEs/V* und ITJ
      ((R:SylStructure.parent.lisp_ustts_pos matches "N.*\\|ITJ")
;      ((R:SylStructure.parent.lisp_ustts_pos matches "N.*\\|V.*\\|ITJ")
;;; nur auf die betonte Silbe
       ((stress is 1)
;;; letztes NN innerhalb kriegt H*L
	((R:SylStructure.parent.lisp_last_NN_in_phrase is 1)
;;; volle Phrasengrenze oder nur ip? 
         ((R:SylStructure.parent.pbreak is "BB")
;;; Fragesatz -> steigender Akzent
	  ((R:SylStructure.parent.R:Token.parent.punc matches "\\?") 
	   (("L*H"))
	   (("H*L")))
	  (("H*L")))
	 (("L*H")))
	((NONE)))
       ((NONE))))


;;;; Feature Functions used in  simple_int_tone_cart_tree
;;;;                        and simple_int_accent_cart_tree


;;; Boundary tones duerfen nur auf die letzte Silbe eines Wortes
;;; gesetzt werden, daher:

(define (last_syl_in_word syl)
"(last_syl_in_word SYL)
 returns 1, if syllable is word final
 0 else"

(cond 
 ((equal? (item.feat syl 'R:SylStructure.parent.name)
	  (item.feat syl 'n.R:SylStructure.parent.name))
  "0")
 (t "1")))


;;; Unterscheidung von phrasenfinalen NNs

(define (last_NN_in_phrase word)
"(last_NN_in_phrase WORD)
 returns "1", if there's no following NN
 within this phrase, "0" else"

(cond
 ((null (item.next word)) "1")
 ((string-matches (item.feat word 'pbreak) "B\\|BB")
  ;; last word in phrase
  "1")
 ((string-matches (item.feat word 'n.pos) "N.*")
  ;; next word is NN
  "0")
 (t (last_NN_in_phrase (item.next word)))))

;; doubled function from ims_german_ling_preproc.scm
(define (first_word_in_sentence syl)
"(first_word_in_sentence SYL)
returns POS-tag of first word of the sentence SYL is in"

(if (string-equal "0" (item.feat syl 'R:SylStructure.parent.R:Word.p.name))
    (item.feat syl 'R:SylStructure.parent.pos)
    (if (string-matches 
	 (item.feat syl 'R:SylStructure.parent.R:Token.parent.p.punc)  
	 "\\.")
	(item.feat syl 'R:SylStructure.parent.pos)
	(first_word_in_sentence (item.relation (item.daughter1 (item.prev (item.parent (item.relation syl 'SylStructure)))) 'Syllable)))))


;;;
;;; Relate phrasing to boundary tones.
;;;

; useless: tobi labels not available for module Phrasify

;(set! tobi_label_phrase_cart_tree
;'
;((R:SylStructure.daughtern.R:Intonation.daughtern.name is "-")
; ((B))
; ((R:SylStructure.daughtern.R:Intonation.daughtern.name matches ".*%")
;  ((BB))
;  ((NB)))))


(set! german_accent_tree
      '
      ((lisp_german_accent is "L*H") 
;;; L*H ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;        
       ((lisp_german_endtone is "NONE")              
	((n.lisp_german_accent is "NONE")
	 ((n.lisp_german_endtone is "NONE")
;; disyllabic ("normal")
	  ((((A 60 0)(N 50 100))))            
;; boundary after following syllable
	  ((n.lisp_german_endtone is "L%")           
	   ((((A 60 0)(N 0 100)(N 100 -25)))) ; L% 
	   ((((N 20 0)(N 100 135))))))        ; -, % or H%
;; interference with accent on following syllable: "monosyllabic"
	 ((((A 30 0)(A 80 100)))))            
;; boundary after this syllable: "monosyllabic"
	((lisp_german_endtone is "L%")
	 ((((ERROR))))                      ; L%
	 ((((A 20 0)(A 100 135))))))          ; -, % or H%
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
       ((lisp_german_accent is "H*L")
;;; H*L ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	((lisp_german_endtone is "NONE")
	 ((n.lisp_german_accent is "NONE")
	  ((n.lisp_german_endtone is "NONE")
;; no interference with following accent or boundary
	   ((lisp_prev_accent is "NONE")
	    ((((A 0 0)(A 60 100)(N 50 0))))
	    ((((A 60 100)(N 50 0)))))           
;; boundary after following syllable
	   ((n.lisp_german_endtone is "H%")
;	    ((((ERROR))))                        ; H%
	    ((((N 20 0)(N 100 135)))) ; i.e. => L*H,H% (Hack to make it more robust)
	    ((n.lisp_german_endtone is "L%")
	     ((lisp_prev_accent is 0)                 ; L%
	      ((((A 0 0)(A 60 100)(N 50 0)(N 100 -25))))
	      ((((A 60 100)(N 50 0)(N 100 -25)))))   
	     ((lisp_prev_accent is 0)                 ; - or %
	      ((((A 0 0)(A 60 100)(N 50 0)(N 100 0))))
	      ((((A 60 100)(N 50 0)(N 100 0))))))))  
;; accent on following syllable: "monosyllabic"
	  ((lisp_prev_accent is 0)
	   ((((A 0 0)(A 30 100)(A 80 0))))
	   ((((A 30 100)(A 80 0)))))) 
;; boundary after this syllable: "monosyllabic"
	 ((lisp_german_endtone is "H%")         
;	  ((((ERROR))))                          ; H%
	  ((((A 20 0)(A 100 135)))) ; i.e. => L*H,H% (Hack to make it more robust)
	  ((lisp_german_endtone is "L%")
	   ((lisp_prev_accent is 0)
	    ((((A 0 0)(A 30 100)(A 100 -25))))
	    ((((A 30 100)(A 100 -25)))))             ; L%
	   ((lisp_prev_accent is 0)
	    ((((A 0 0)(A 30 100)(A 80 0)(A 100 0))))
	    ((((A 30 100)(A 80 0)(A 100 0))))))))   ; - or %
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	((lisp_german_accent is "L*!H") 
;;; L*!H ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;        
	 ((lisp_german_endtone is "NONE")              
	  ((n.lisp_german_accent is "NONE")
	   ((n.lisp_german_endtone is "NONE")
;; "normal"
	    ((((A 60 0)(N 50 DSTEP))))            
;; boundary after following syllable
	    ((n.lisp_german_endtone is "L%")           
	     ((((A 60 0)(N 0 DSTEP)(N 100 -25)))) ; L% 
	     ((((N 20 0)(N 100 135))))))        ; -, % or H% --> kein Downstep?
;; accent after following syllable: "monosyllabic"
	   ((((A 30 0)(A 80 DSTEP)))))            
;; boundary after this syllable: "monosyllabic"
	  ((lisp_german_endtone is "L%")
	   ((((ERROR))))                      ; L%
	   ((((A 20 0)(A 100 135))))))          ; -, % or H% --> kein Downstep?
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	 ((lisp_german_accent is "!H*L")
;;; !H*L ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;   
	  ((lisp_german_endtone is "NONE")
	   ((n.lisp_german_accent is "NONE")
	    ((n.lisp_german_endtone is "NONE")
;; "normal"
	     ((((A 60 DSTEP)(N 50 0))))           
;; boundary after following syllable
	     ((n.lisp_german_endtone is "H%")
	      ((((ERROR))))                        ; H%
	      ((n.lisp_german_endtone is "L%")
	       ((((A 60 DSTEP)(N 50 0)(N 100 -25))))   ; L% 
	       ((((A 60 DSTEP)(N 50 0)(N 100 0)))))))  ; - or %
;; accent after following syllable: "monosyllabic"
	    ((((A 30 DSTEP)(A 80 0))))) 
;; boundary after this syllable
	   ((lisp_german_endtone is "H%")         
	    ((((ERROR))))                          ; H%
	    ((lisp_german_endtone is "L%")
	     ((((A 30 DSTEP)(A 100 -25))))             ; L%
	     ((((A 30 DSTEP)(A 80 0)(A 100 0)))))))   ; - or %
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	  ((lisp_german_accent is "L*")
;;; L* ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	   ((((A 60 0))))
	   ((lisp_german_accent is "H*")
;;; H* ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	    ((lisp_prev_accent is 0)
	     ((((A 0 0)(A 60 100))))
	     ((((A 60 100)))))                            ; kein Downstep?
	    ((lisp_german_accent is "..H")
;;; ..H ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	     ((((A 50 100))))
	     ((lisp_german_accent is "..L")
;;; ..L ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	      ((((A 60 0))))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	      ((lisp_german_accent is "L*HL")
;;; L*HL ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	       ((lisp_german_endtone is "NONE")
		((n.lisp_german_accent is "NONE")         
		 ((n.lisp_german_endtone is "NONE")       
;; "normal" case
		  ((syl_coda_type is "-V") 
		   ((((A 30 0)(A 100 100)(N 90 0))))
		   ((n.syl_onset_type is "-V") 
		    ((((A 30 0)(A 100 100)(N 90 0))))
		    ((((A 30 0)(N 10 100)(N 90 0))))))
;; boundary after following syllable
		  ((n.lisp_german_endtone is "L%")
		   ((syl_coda_type is "-V") 
		    ((((A 30 0)(A 100 100)(N 90 -25))))
		    ((n.syl_onset_type is "-V") 
		     ((((A 30 0)(A 100 100)(N 90 -25))))
		     ((((A 30 0)(N 10 100)(N 90 -25))))))
		   ((n.lisp_german_endtone is "H%")
		    ((((ERROR))))
		    ((syl_coda_type is "-V") 
		     ((((A 30 0)(A 100 100)(N 90 0)(N 100 0))))
		     ((n.syl_onset_type is "-V") 
		      ((((A 30 0)(A 100 100)(N 90 0)(N 100 0))))
		      ((((A 30 0)(N 10 100)(N 90 0)(N 100 0)))))))))
;; accent on following syllable
		 ((((ERROR)))))
;; boundary after this syllable
		((((ERROR)))))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	       ((lisp_german_accent is "HH*L")
;;; HH*L ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		((p.lisp_german_accent is "NONE")
		 ((pp.lisp_german_accent is "NONE")
		  ((lisp_german_endtone is "NONE")
		   ((n.lisp_german_accent is "NONE")
		    ((n.lisp_german_endtone is "NONE")
;; "normal" case: no interference with accents or boundaries
		     ((lisp_prev_accent is 0)
		      ((((P 0 0)(P 50 100)(A 50 70)(N 30 0))))     ; eigentlich Target auf Baseline am Anfang d. letzten Silbe, nicht bei Voicing!
		      ((((P 50 100)(A 50 70)(N 30 0)))))
;; boundary after following syllable
		     ((n.lisp_german_endtone is "L%")
		      ((lisp_prev_accent is 0)
		       ((((P 0 0)(P 50 100)(A 50 70)(N 30 0)(N 100 -25))))
		       ((((P 50 100)(A 50 70)(N 30 0)(N 100 -25)))))
		      ((n.lisp_german_endtone is "H%")
		       ((((ERROR))))
		       ((lisp_prev_accent is 0)
			((((P 0 0)(P 50 100)(A 50 70)(N 30 0)(N 100 0))))
			((((P 50 100)(A 50 70)(N 30 0)(N 100 0))))))))
;; accent on following syllable
		    ((lisp_prev_accent is 0)
		     ((((P 0 0)(P 50 100)(P 90 70)(A 0 0))))
		     ((((P 50 100)(P 90 70)(A 0 0))))))
;; boundary after this syllable
		   ((lisp_german_endtone is "L%")
		    ((lisp_prev_accent is 0)
		     ((((P 0 0)(P 20 100)(P 90 70)(A 0 0)(A 100 -25))))
		     ((((P 20 100)(P 90 70)(A 0 0)(A 100 -25)))))
		    ((lisp_german_endtone is "H%")
		     ((((ERROR))))
		     ((lisp_prev_accent is 0)
		      ((((P 0 0)(P 20 100)(P 90 70)(A 0 0)(A 100 0))))
		      ((((P 20 100)(P 90 70)(A 0 0)(A 100 0))))))))    
;; accent on preceding syllables
		  ((((ERROR)))))
		 ((((ERROR)))))
		((lisp_german_accent is "NONE")
		 ((((NONE))))
;; any other accents: not implemented
		 ((((UNKNOWN))))))))))))))))


(set! german_endtone_tree
;; there are only values specified for those boundaries that
;; are not immediately preceeded by an accent. 
;; boundaries preceded by an accent are already specified 
;; by german_accent_tree, see above.
      '
      ((lisp_german_endtone in ("%" "-"))
       ((p.lisp_german_accent is "NONE")
	((pp.lisp_german_accent is "NONE")
	 ;; f0-targets for boundary not processed yet
	 ((lisp_prev_accent matches ".*H")
	  ((((A 100 100))))
	  ((((A 100 0)))))))      
; no else-condition, targets already specified by german_accent_tree
       ((lisp_german_endtone is "L%")
	((p.lisp_german_accent is "NONE") 
	 ((pp.lisp_german_accent is "NONE")
	  ;; f0-targets not processed yet
	  ((((A 0 0)(A 100 -25)))))) 
; no else-condition, targets already specified by german_accent_tree
	((lisp_german_endtone is "H%");
	 ((p.lisp_german_accent is "NONE") 
	  ((pp.lisp_german_accent is "NONE")
	   ((lisp_prev_accent matches ".*L")
	    ((((A 0 0)(A 100 135))))
	    ((((A 0 100)(A 100 135)))))))
	 ((lisp_german_endtone is "NONE")
	  ((((NONE))))
	  ((((UNKNOWN)))))))))

	    
           

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; New feature functions

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


; jetzt in "ims_extra_features.scm" !
;(define (prev_accent syl)  ; AWB's last_accent_type liefert auch Boundaries
;  "(prev_accent SYL)
;  returns previous accent (no boundaries, just real accents!)"
;  (if (or                            ;; syllable is not accented
;       (null (item.prev syl)) 
;       (null (equal? (item.feat syl 'p.tobi_endtone) "NONE")))
;      "NONE"                         ; no preceding syllables within ip
;      (if (not (equal? "NONE" (item.feat syl 'p.tobi_accent)))
;	  (item.feat syl 'p.tobi_accent)
;	  (prev_accent (item.prev syl))))) ; check preceding syllables
 

(define (german_endtone syl)
 "(german_endtone SYL)
 returns german tobi endtone, assuming that there is no more than 
 one accent and one endtone per syllable"
 (if (eq 0 (item.feat syl 'R:Intonation.daughter1.name))
     "NONE"                              ; neither endtone nor accent
     (if 
      (string-matches 
       (item.feat syl 'R:Intonation.daughter1.name) 
       "\\([HL]?%\\)\\|-")
                                         ; daughter1 is an endtone
      (item.feat syl 'R:Intonation.daughter1.name)
      (if 
       (eq 0 (item.feat syl 'R:Intonation.daughter2.name))
       "NONE"
       (if 
	(string-matches 
	 (item.feat syl 'R:Intonation.daughter2.name)
	 "\\([HL]?%\\)\\|-")
                                         ; daughter2 is an endtone
       (item.feat syl 'R:Intonation.daughter2.name)
       "NONE")))))

;(define (german_endtone word)
;
;(if (eq 0 (item.feat word 'R:SylStructure.daughtern.R:Intonation.name))
;    "NONE"
;    (if (string-matches (item.feat word 'R:SylStructure.daughtern.german_endtone) ".*-\\|.*%")
;	(item.feat word 'R:SylStructure.daughtern.german_endtone)
;	"NONE")))


;  (if 
;   (eq 0 (item.feat word 'R:SylStructure.daughtern.R:Intonation.daughter1.name))
;   "NONE"                              ; neither endtone nor accent
;   (if 
;    (string-matches 
;     (item.feat word 'R:SylStructure.daughtern.R:Intonation.daughter1.name) 
;       "\\([HL]?%\\)\\|-")
;                                         ; daughter1 is an endtone
;    (item.feat word 'R:Intonation.daughter1.name)
;    (if 
;     (eq 0 (item.feat word 'R:SylStructure.daughtern.R:Intonation.daughter2.name))
;     "NONE"
;     (if 
;      (string-matches 
;       (item.feat word 'R:SylStructure.daughtern.R:Intonation.daughter2.name)
;       "\\([HL]?%\\)\\|-")
;      ; daughter2 is an endtone
;      (item.feat word 'R:SylStructure.daughtern.R:Intonation.daughter2.name)
;      "NONE")))))


(define (german_accent syl)
 "(german_accent SYL)
 returns german tobi accent, assuming that there is no more than 
 one accent and one endtone per syllable"
 (if 
  (eq 0 (item.feat syl 'R:Intonation.daughter1.name))
  "NONE"                              ; syllable is not accented
  (if 
   (string-matches 
    (item.feat syl 'R:Intonation.daughter1.name) 
    "\\(L\\*!?HL?\\)\\|\\(H?!?H\\*L\\)\\|\\([LH]\\*\\)\\|\\(\\.\\.[LH]\\)")
                                         ; daughter1 is an accent
   (item.feat syl 'R:Intonation.daughter1.name)
   (if 
    (eq 0 (item.feat syl 'R:Intonation.daughter2.name))
    "NONE"
    (if 
     (string-matches 
      (item.feat syl 'R:Intonation.daughter2.name)
      "\\(L\\*!?HL?\\)\\|\\(H?!?H\\*L\\)\\|\\([LH]\\*\\)\\|\\(\\.\\.[LH]\\)")
                                              ; daughter2 is an accent
     (item.feat syl 'R:Intonation.daughter2.name)
     "NONE")))))


(define (distance_to_last_endtone syl)
 "(distance_to_last_endtone SYL)
 returns number of syllables from last endtone to this syllable,
 0 if there's an endtone on syl or if it's the first item in the utterance"

(if
 (or (null (item.prev syl)) 
     (string-matches (german_endtone syl) "%\\|-"))
 0
 (+ 1 (distance_to_last_endtone (item.prev syl)))))

(define (pos_after_next_break syl)
"(pos_after_next_break SYL)
returns POS-tag of first word after the next break following SYL"

(if (null (item.next (syl_with_next_break syl))) 
    "NONE"
    (item.feat (item.parent (item.relation (item.next (syl_with_next_break syl)) 'SylStructure)) 'pos)))
 

(define (syl_with_next_break syl)
"(syl_with_next_break SYL)
return first syl in word with next break"
  (if
   (null (item.next syl))
   syl
   (if (string-matches 
	(item.feat syl 'R:SylStructure.parent.pbreak) "^B.*")
       (item.relation (item.daughtern (item.parent (item.relation syl 'SylStructure))) 'Syllable)
       (syl_with_next_break (item.next syl)))))

(provide 'ims_german_tobi_rules)



